#include <cstdio>
#include <cstdlib>
#include <ctime>

#include <iostream>
#include <queue>
#include <set>

#include "Matrix.hpp"

using namespace std;

unsigned DMatrix::GetXDim()
{
  return m_uiXDim;
}

unsigned DMatrix::GetYDim()
{
  return m_uiYDim;
}

/*************** Array Manipulationen an der Groesse ****************/
int DMatrix::Init(unsigned uiXDim, unsigned uiYDim)
{
  //Zufallsgenerator initialisieren
  srand(time(NULL));

  //Neue Werte ueberpruefen!
  if (uiXDim == 0 || uiYDim == 0)
    return INVALID_SIZE;

  //Alte Matrix entfernen
  m_cMatrix.clear();

  //Neue Matrix erstellen und initialisieren
  m_uiXDim = uiXDim;
  m_uiYDim = uiYDim;
  m_cMatrix=vector<Column>(m_uiXDim);
  for (unsigned x = 0; x < m_uiXDim; x++)
    m_cMatrix[x]=Column(m_uiYDim);

  return 1;
}

/* Diese Funktion fuegt in eine bestehende Matrix eine weitere Spalte ein, dies
   funktioniert jedoch nur dann, wenn bereits eine Matrix existiert, da die
   Groesse der einzufuegenden Spalte nicht aus den Daten ermittelt werden
   kann*/
int DMatrix::InsertCol(unsigned uiX, Column pcCol)
{
  //uiX gibt die Position an, an dem die neue Spalte nach dem Einfuegen
  //steht

  //erlaubter Wertebereich: 0 bis m_uiXDim
  if (uiX > m_uiXDim)
    return INDEX_OUT_OF_BOUNDS;

  //Falls YDim == 0 ist, macht das keinen Sinn!
  if (m_uiYDim == 0)
    return MATRIX_INVALID;

  //Groesse der Spalte testen
  if (pcCol.size() != m_uiYDim)
    return INDEX_OUT_OF_BOUNDS;

  //Neue Spalte einfuegen
  //m_cMatrix.resize(m_uiXDim+1);
  m_uiXDim++;

  //insert resized automatisch
  m_cMatrix.insert(m_cMatrix.begin()+uiX, 1, pcCol);

  return 1;
}

/* Diese Funktion fuegt in eine bestehende Matrix eine weitere Zeile ein, dies
   funktioniert jedoch nur dann, wenn bereits eine Matrix existiert, da die
   Groesse der einzufuegenden Zeile nicht aus den Daten ermittelt werden
   kann*/
int DMatrix::InsertRow(unsigned uiY, Row pcRow)
{
  //uiY gibt die Position an, an dem die neue Zeile nach dem Einfuegen
  //steht

  //erlaubter Wertebereich: 0-m_uiYDim
  if (uiY > m_uiYDim)
    return INDEX_OUT_OF_BOUNDS;

  //Falls YDim == 0 ist, macht das keinen Sinn!
  if (m_uiXDim == 0)
    return MATRIX_INVALID;

  //Jede Spalte um eins vergroessern
  m_uiYDim++;
  for (unsigned uiCol = 0; uiCol < m_uiXDim; ++uiCol)
  {
    //Einfuegen - insert macht automatisch ein resize
    m_cMatrix[uiCol].insert(m_cMatrix[uiCol].begin()+uiY, 1, pcRow[uiCol]);
  }

  return 1;
}

//Entferne Spalte
int DMatrix::RemoveCol(unsigned uiX)
{
  //Ist der Index im richtigen Bereich?
  if (uiX >= m_uiXDim)
    return INDEX_OUT_OF_BOUNDS;

  m_cMatrix.erase(m_cMatrix.begin()+uiX);
  m_uiXDim--;
  m_cMatrix.resize(m_uiXDim);

  //Fertig
  return 1;
}

//Entferne Zeile
int DMatrix::RemoveRow(unsigned uiY)
{
  //Ist der Index im gueltigen Bereich?
  if (uiY >= m_uiYDim)
    return INDEX_OUT_OF_BOUNDS;

  m_uiYDim--;
  for (unsigned uiCol = 0; uiCol < m_uiXDim; uiCol++)
  {
    m_cMatrix[uiCol].erase(m_cMatrix[uiCol].begin()+uiY);
    m_cMatrix[uiCol].resize(m_uiYDim);
  }

  return 1;
}

DMatrix* DMatrix::DoPermutation(int* piPerm)
{
  DMatrix* pcRet;
  Column pcCol(m_uiYDim);

  //Permutation umsetzen
  pcRet = new DMatrix(m_uiXDim, m_uiYDim);
  for (unsigned i = 0;i<m_uiXDim; i++)
  {
    int iRet;
    GetCol(i, iRet, pcCol);
    //todo: iRet auswerten
    pcRet->SetCol(piPerm[i], pcCol);
  }

  return pcRet;
}

//Liefert eine Permutation der Spalten der Matrix
DMatrix* DMatrix::Permute()
{
  DMatrix* pcRet;
  int* piPerm;
  int iRand;

  if (m_uiXDim == 0)
    return NULL;

  //Array fuer Permutation erstellen
  piPerm = new int[m_uiXDim];

  //Array initialisieren
  for (unsigned i=0;i<m_uiXDim;i++)
    piPerm[i] = -1;

  //Permutation erzeugen
  for (unsigned i = 0; i < m_uiXDim; i++)
  {
    unsigned c;
    //Die wievielte Position soll geaendert werden?
    iRand = rand() % (m_uiXDim - i);

    //Position suchen
    for (c = 0; iRand >= 0; c++)
    {
      //cout << "c=" << c << endl;
      //cout << "uiRand" << uiRand << endl;
      if (piPerm[c] == -1)
        iRand--;
    }
    //c zeigt nun auf die Position (eigentlich eins weiter)
    //Permutation setzen
    piPerm[c-1]=i;
  }
  pcRet = DoPermutation(piPerm);
  delete[] piPerm;

  return pcRet;
}


/********************** Zugriff auf Elemente ************************/
int DMatrix::SetElem(unsigned uiX, unsigned uiY, int iValue)
{
  //Matrix ungueltig?
  if (m_uiXDim == 0 || m_uiYDim == 0)
    return MATRIX_INVALID;

  //Indizes ok?
  if (uiX >= m_uiXDim || uiY >= m_uiYDim)
    return INDEX_OUT_OF_BOUNDS;

  m_cMatrix[uiX][uiY] = iValue;

  return 1;
}

int DMatrix::GetElem(unsigned uiX, unsigned uiY, int& iValue)
{
  //Matrix ungueltig?
  if (m_uiXDim == 0 || m_uiYDim == 0)
    return MATRIX_INVALID;

  //Indizes ok?
  if (uiX >= m_uiXDim || uiY >= m_uiYDim)
    return INDEX_OUT_OF_BOUNDS;

  iValue = m_cMatrix[uiX][uiY];

  return 1;
}

Row DMatrix::GetRow(unsigned uiY, int& iRetValue, Row& rcRow)
{
  rcRow.clear();

  //Matrix ungueltig?
  if (m_uiXDim == 0 || m_uiYDim == 0)
  {
    iRetValue = MATRIX_INVALID;
    return rcRow;
  }

  //Index der Zeile gueltig?
  if (m_uiYDim <= uiY)
  {
    iRetValue = INDEX_OUT_OF_BOUNDS;
    return rcRow;
  }

  //Groesse setzen
  rcRow.reserve(m_uiYDim);

  //Zeile auslesen
  for (unsigned x = 0; x < m_uiXDim; x++)
    rcRow.push_back(m_cMatrix[x][uiY]);

  iRetValue = 1;

  return rcRow;
}

Column DMatrix::GetCol(unsigned uiX, int& iRetValue, Column& rcCol)
{
  //Matrix ungueltig?
  if (m_uiXDim == 0 || m_uiYDim == 0)
  {
    iRetValue = MATRIX_INVALID;
    cout << "Matrix invalid" << endl;
    return rcCol;
  }

  //Index der Zeile gueltig?
  if (m_uiXDim <= uiX)
  {
    iRetValue = INDEX_OUT_OF_BOUNDS;
    cout << "Index out of Bounds" << endl;
    return rcCol;
  }

  rcCol = m_cMatrix[uiX];

  iRetValue = 1;

  return rcCol;
}

Column DMatrix::GetCol(unsigned uiX, int& iRetValue)
{
  Column cCol;

  return GetCol(uiX, iRetValue, cCol);
}

Row DMatrix::GetRow(unsigned uiY, int& iRetValue)
{
  Row cRow;

  return GetRow(uiY, iRetValue, cRow);
}

int DMatrix::SetCol(unsigned uiX, Column cCol)
{
  //Matrix ueberhaupt gueltig?
  if (m_uiXDim == 0 || m_uiYDim == 0)
    return MATRIX_INVALID;

  //Index der Spalte gueltig?
  if (m_uiXDim <= uiX)
    return INDEX_OUT_OF_BOUNDS;

  //Die neue Spalte muss die gleiche Groesse wie die Matrix haben, sonst
  //funktioniert das hier nicht
  if (cCol.size() != m_uiYDim)
    return INVALID_SIZE;

  m_cMatrix[uiX] = cCol;

  return 1;
}

int DMatrix::SetRow(unsigned uiY, Row cRow)
{
  //Matrix ueberhaupt gueltig?
  if (m_uiXDim == 0 || m_uiYDim == 0)
    return MATRIX_INVALID;

  //Index der Zeile gueltig?
  if (m_uiYDim <= uiY)
    return INDEX_OUT_OF_BOUNDS;

  //Die neue Zeile muss die gleiche Groesse wie die Matrix haben, sonst
  //funktioniert das hier nicht
  if (cRow.size() != m_uiXDim)
    return INVALID_SIZE;

  for (unsigned x = 0; x < m_uiXDim; x++)
    m_cMatrix[x][uiY] = cRow[x];

  return 1;
}

DMatrix::~DMatrix()
{
}

DMatrix::DMatrix()
{
  m_uiXDim = m_uiYDim = 0;
}

DMatrix::DMatrix(DMatrix &rcMatrix)
{
  //Initialisieren
  m_uiXDim = m_uiYDim = 0;

  //Matrix kopieren
  if (rcMatrix.m_uiXDim > 0 && rcMatrix.m_uiYDim > 0)
  {
    m_uiXDim = rcMatrix.m_uiXDim;
    m_uiYDim = rcMatrix.m_uiYDim;

    m_cMatrix = rcMatrix.m_cMatrix;
  }
}

DMatrix::DMatrix(unsigned uiXDim, unsigned uiYDim)
{
  //Initialisieren
  m_uiXDim = m_uiYDim = 0;

  //Werte gueltig?
  if (uiXDim == 0 || uiYDim == 0)
    return;

  //Werte kopieren
  m_uiXDim = uiXDim;
  m_uiYDim = uiYDim;

  //Array initialisieren
  m_cMatrix = vector<Column>(m_uiXDim);
  for (unsigned x = 0; x < m_uiXDim; x++)
    m_cMatrix[x]=Column(m_uiYDim);
}

//Algorithmus
//geg. Permutation P
//letzte Zahl in sortierte Liste einfuegen
//ist die vorletzte Zahl kleiner als die in der sortierten Liste
//dann die naechstgroessere als vorletzte zahl nehmen - den rest sortiert
//einfuegen (kleinere zu erst)
//ansonsten vorvorletzte Zahl pruefen ...
int DMatrix::NextPerm(int* piPerm, int iSize)
{
  set<int> liList;
  set<int>::iterator itList;

  liList.insert(piPerm[iSize-1]);

  for (int i=iSize-2; i >= 0; i--)
  {
    liList.insert(piPerm[i]);
    //Gibt es groessere Elemente als den aktuellen in der Liste?
    itList = liList.upper_bound(piPerm[i]);

    if (itList != liList.end())
    { //Gefunden!
      //Element einfuegen
      piPerm[i] = *itList;

      //Element entfernen
      liList.erase(*itList);

      //Alle weiteren Elemente einfuegen
      itList=liList.begin();
      for (i = i + 1; i < iSize; i++)
      {
        piPerm[i] = *itList;
        ++itList;
      }

      return 1;
    }
  }

  //Keine weitere Permutation gefunden
  return 0;
}

std::vector<DMatrix*> DMatrix::PermuteAll(int& riError)
{
  int* piPerm;
  vector<DMatrix*> vRet;

  if (m_uiXDim == 0)
  {
    riError = MATRIX_INVALID;
    return vRet;
  }

  //Identische Permutation anlegen
  piPerm = new int[m_uiXDim];
  for (unsigned i = 0; i < m_uiXDim; i++)
    piPerm[i] = i;

  //Ueber alle Permutationen iterieren
  do
  {
    vRet.push_back(DoPermutation(piPerm));
  } while(NextPerm(piPerm, m_uiXDim));

  delete[] piPerm;
  return vRet;
}

int DMatrix::AddSka(int iValue)
{
  if (m_uiXDim == 0 || m_uiYDim == 0)
    return MATRIX_INVALID;

  for (unsigned x = 0; x < m_uiXDim; ++x)
  {
    for (unsigned y = 0; y < m_uiYDim; ++y)
    {
      m_cMatrix[x][y] += iValue;
    }
  }

  return 1;
}

int DMatrix::MulSka(int iValue)
{
  if (m_uiXDim == 0 || m_uiYDim == 0)
    return MATRIX_INVALID;

  for (unsigned x = 0; x < m_uiXDim; ++x)
  {
    for (unsigned y = 0; y < m_uiYDim; ++y)
    {
      m_cMatrix[x][y] *= iValue;
    }
  }

  return 1;
}

void DMatrix::Show()
{
  if (m_cMatrix.empty())
    cerr << "Matrix ist leer!" << endl;

  for (unsigned y = 0; y < m_cMatrix[0].size(); y++)
  {
    for (unsigned x = 0; x < m_cMatrix.size(); x++)
    {
      cerr << m_cMatrix[x][y] << " ";
    }
    cerr << endl;
  }
}

//Die Anzahl der Spalten muss gleich sein
int DMatrix::ConcatRows(DMatrix& rcMatrix)
{
  Row cRow;
  int iFehler;

  if (rcMatrix.m_uiXDim != m_uiXDim)
    return INVALID_SIZE;

  for (unsigned i = 0; i < rcMatrix.m_uiYDim; i++)
  {
    cRow = rcMatrix.GetRow(i, iFehler, cRow);
    InsertRow(m_uiYDim, cRow);
  }

  return 1;
}

//Die Anzahl der Zeilen muss gleich sein
int DMatrix::ConcatCols(DMatrix& rcMatrix)
{
  Column cCol;
  int iFehler;

  if (rcMatrix.m_uiYDim != m_uiYDim)
    return INVALID_SIZE;

  for (unsigned i=0;i<rcMatrix.m_uiXDim; i++)
  {
    cCol = rcMatrix.GetCol(i, iFehler, cCol);
    InsertCol(m_uiXDim, cCol);
  }

  return 1;
}

/*int main(int argc, char* argv[])
{
  int iFehler;
  srand(time(NULL));
  //Matrix erstellen
  DMatrix test(3, 4);
  DMatrix test2(5, 4);
  DMatrix test3;

  test3.Init(3, 3);
  vector<DMatrix*> pcAllPerms;
  Column c1, c2, c3;

  //Matrix mit Werten fuellen
  for (unsigned x = 0; x < 3; x++)
  {
    for (unsigned y = 0; y < 4; y++)
      test.SetElem(x, y, x+y);
  }

  for (unsigned x = 0; x < 5; x++)
  {
    for (unsigned y = 0; y < 4; y++)
      test2.SetElem(x, y, x+y);
  }

  for (unsigned x = 0; x < 3; x++)
  {
    for (unsigned y = 0; y < 3; y++)
      test3.SetElem(x, y, x+y);
  }

  //Mal alle anzeigen
  test.Show();
  cout << "--------------" << endl;
  test2.Show();
  cout << "--------------" << endl;
  test3.Show();
  cout << "--------------" << endl;

  test2.ConcatCols(test);
  test2.Show();
  cout << "--------------" << endl;
  test3.ConcatRows(test);
  test3.Show();
  return 0;
}*/
