/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,   *
 *   kermit@frog   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gas.h"

#include <algorithm>
#include <iostream>

using namespace std;

GAS::GAS()
{
}

GAS::GAS(GAS& g)
{
  m_G0 = g.m_G0;
}

GAS::~GAS()
{
}

void GAS::Clear()
{
  m_G0.clear();
}

//1  => ok
//0  => GENERIC ERROR
//-1 => not all participants are used
int GAS::Check()
{
  unsigned uiUsers = Max(m_G0);
  SubSet sAll;
  set<SubSet>::iterator i;
  SubSet::iterator i2;

  //Mal alle Elemente zusammenfassen
  for (i = m_G0.begin(); i != m_G0.end(); ++i)
  {
    for (i2 = i->begin(); i2 != i->end(); ++i2)
      sAll.insert(*i2);
  }

  //Kommen alle Elemente wirklich vor?
  if (uiUsers != sAll.size())
    return -1;

  return 1;
}

//1 => OK
//0 => GENERIC ERROR
//-1 => isolated participant

int GAS::AddQualifiedSet(SubSet vSet)
{
  set<SubSet>::iterator i;

  //Check for isolated participant
  if (vSet.size() < 2)
    return -1;

  //Ist diese Menge eventuell ein neuer Kandidat fuer m_G0?
  //Dies trifft nur dann nicht zu, falls eine bereits enthaltene
  //Menge, Untermenge von uiSet ist
  //Sollte sich herausstellen, dass die Menge eingefuegt wird,
  //so muessen alle Obermengen dieser Menge aus m_G0 entfernt
  //werden
  for (i=m_G0.begin(); i!= m_G0.end(); ++i)
  {
    if (IsSubSetEq(*i, vSet))
      return 1; //Nicht einfuegen
  }

  //Menge wird neu eingefuegt werden, davor
  //Alle rausschmeissen die Obermenge sind
  for (i=m_G0.begin(); i!=m_G0.end(); ++i)
  {
    if (IsSubSetEq(vSet, *i))
      m_G0.erase(i);
  }

  //Menge einfuegen
  m_G0.insert(vSet);

  return 1;
}

//Diese Funktion erstellt die maximal verbotenen Mengen
//aus den minimal qualifizierten Mengen
set<SubSet> GAS::BuildMaxForbidden()
{
  set<SubSet>::iterator i1, i2;
  set<SubSet> Ftemp;
  set<SubSet> FM;
  SubSet sTemp;
  unsigned uiUsers = Max(m_G0);

  //FM enthaelt zunaechst alle Teilnehmer
  for (unsigned c = 1; c <= uiUsers; c++)
    sTemp.insert(c);
  FM.insert(sTemp);

  for (i1 = m_G0.begin(); i1 != m_G0.end(); ++i1)
  {
    Ftemp.clear();
    //Alle Elemente von FM finden, die Obermenge der Teilmenge *i1 sind
    for (i2 = FM.begin(); i2 != FM.end(); ++i2)
    {
      if (IsSubSetEq(*i1, *i2))
      { //Diese Elemente aus FM entfernen und in Ftemp speichern
        Ftemp.insert(*i2);
        FM.erase(i2);
      }
    }

    //FTemp enthaelt nun alle Obermengen von *i1
    for(i2=Ftemp.begin(); i2 != Ftemp.end(); ++i2)
    {
      SubSet::iterator i3;
      SubSet::iterator itDelete2;
      for (i3 = i1->begin(); i3 != i1->end(); ++i3)
      { //Alle Bi Mengen bilden - jedes Element aus *i1 entfernen
        sTemp = *i2;
        sTemp.erase(*i3);
        //Bi Mengen in neue MaxForbidden einfuegen
        AddToMaxForbidden(FM, sTemp);
      }
    }
  }


  return FM;
}

//Die Menge wird eingefuegt, falls keine der enthaltenen Mengen
//Obermenge dieser Menge ist, danach muessen alle Teilmengen
//dieser Menge entfernt werden
void GAS::AddToMaxForbidden(set<SubSet>& FM, SubSet sSet)
{
  set<SubSet>::iterator i;

  for (i = FM.begin(); i != FM.end(); ++i)
  {
    if (IsSubSetEq(sSet, *i))
      return; //Nicht einfuegen
  }

  //Menge wird neu eingefuegt werden, davor
  //Alle rausschmeissen die Teilmenge sind
  for (i = FM.begin(); i != FM.end(); ++i)
  {
    if (IsSubSetEq(*i, sSet))
      FM.erase(i);
  }

  //Menge einfuegen
  FM.insert(sSet);
}

//Ist Set1 Teilmenge von Set2?
bool GAS::IsSubSetEq(const SubSet& sSet1, const SubSet& sSet2)
{
  return includes(sSet2.begin(), sSet2.end(), sSet1.begin(), sSet1.end());
}

//Ist Set1 echte Teilmenge von Set2?
bool GAS::IsSubSet(const SubSet& sSet1, const SubSet& sSet2)
{
  return IsSubSetEq(sSet1, sSet2) && !(sSet1==sSet2);
}

DMatrix* GAS::GetCA()
{
  set<SubSet> FM;
  set<SubSet>::iterator Fj;
  SubSet sTemp;
  unsigned uiUser;
  DMatrix* pcCA;
  unsigned i,j;

  //Anzahl der Teilnehmer holen
  uiUser = Max(m_G0);

  //Maximal verbotene Menge beschaffen
  FM = BuildMaxForbidden();

  //Matrix anlegen
  pcCA = new DMatrix(FM.size(), uiUser);
  for (i = 0; i <= uiUser; i++)
  {
    sTemp.clear();
    sTemp.insert(i+1);
    for (j=0,Fj = FM.begin(); Fj != FM.end(); ++Fj,j++)
    { //Ueber
      if (!IsSubSetEq(sTemp, *Fj))
        pcCA->SetElem(j, i, 1);
      else
        pcCA->SetElem(j, i, 0);
    }
  }

  return pcCA;
}

void GAS::Print(const SubSet& sSet)
{
  SubSet::iterator i;

  for (i=sSet.begin(); i != sSet.end(); ++i)
  {
    cerr << *i << " ";
  }
  cerr << endl;
}

void GAS::Print(const set<SubSet>& sSet)
{
  set<SubSet>::iterator i;

  for (i=sSet.begin(); i != sSet.end(); ++i)
  {
    Print(*i);
  }
}

void GAS::Print(DMatrix* matrix)
{
  int value;

  for (unsigned y = 0; y < matrix->GetYDim(); y++)
  {
    for (unsigned x = 0; x < matrix->GetXDim(); x++)
    {
      matrix->GetElem(x, y, value);
      cerr << value << " ";
    }
    cerr << endl;
  }
}

void GAS::Test()
{
  SubSet s1, s2, s3, s4;
  set<SubSet> FM;
  set<SubSet>::iterator i;
  DMatrix* pcCA;

  //SubSets konstruieren
  s1.insert(1);
  s1.insert(2);

  s2.insert(1);
  s2.insert(3);

  s3.insert(1);
  s3.insert(4);

  s4.insert(2);
  s4.insert(3);
  s4.insert(4);

  //Minimale Mengen konstruieren
  AddQualifiedSet(s1);
  AddQualifiedSet(s2);
  AddQualifiedSet(s4);
  AddQualifiedSet(s3);

  pcCA = GetCA();
  Print(pcCA);
}

unsigned GAS::Max(const SubSet& sSet)
{
  SubSet::iterator i;
  unsigned uiMax = 0;

  for (i = sSet.begin(); i != sSet.end(); ++i)
  {
    if (uiMax < *i)
      uiMax = *i;
  }

  return uiMax;
}

unsigned GAS::Max(const set<SubSet>& sSet)
{
  set<SubSet>::iterator i;
  unsigned uiTemp, uiMax = 0;

  for (i = sSet.begin(); i != sSet.end(); ++i)
  {
    uiTemp = Max(*i);
    if (uiTemp > uiMax)
      uiMax = uiTemp;
  }

  return uiMax;
}

unsigned GAS::GetUser()
{
  return Max(m_G0);
}
