/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gasgui.h"
#include "gasvcs.h"

#include <iostream>
#include <algorithm>

using namespace std;

GasGUI::GasGUI()
{
  QVBoxLayout* mainLayout   = new QVBoxLayout(this);
  QHBoxLayout* fileLayout   = new QHBoxLayout;
  QHBoxLayout* addLayout    = new QHBoxLayout;
  QHBoxLayout* listLayout   = new QHBoxLayout;
  QHBoxLayout* squareLayout = new QHBoxLayout;

  //Buttons erzeugen
  QPushButton* openButton  = new QPushButton(tr("Open"));
  QPushButton* addButton   = new QPushButton(tr("Add"));
  QPushButton* delButton   = new QPushButton(tr("Remove"));
  QPushButton* shareButton = new QPushButton(tr("Share"));
  QPushButton* quitButton  = new QPushButton(tr("Cancel"));

  //Controls erzeugen
  m_pcFileName    = new QLineEdit;
  m_pcAddSet      = new QLineEdit;
  m_pcQualSets    = new QListWidget;
  m_pcForceSquare = new QCheckBox;

  //Top Layouts schachteln
  mainLayout->addLayout(fileLayout);
  mainLayout->addLayout(addLayout);
  mainLayout->addLayout(listLayout);
  mainLayout->addLayout(squareLayout);

  //fileLayout aufbauen
  QVBoxLayout* file1Layout = new QVBoxLayout;
  QVBoxLayout* file2Layout = new QVBoxLayout;
  fileLayout->addLayout(file1Layout);
  fileLayout->addLayout(file2Layout);

  file1Layout->addWidget(new QLabel(tr("File to share")));
  file1Layout->addWidget(m_pcFileName);
  file1Layout->addWidget(new QLabel(tr("Qualified set")));
  file2Layout->addStretch();
  file2Layout->addWidget(openButton);
  file2Layout->addStretch();

  //AddLayout aufbauen
  QVBoxLayout* add1Layout = new QVBoxLayout;
  QVBoxLayout* add2Layout = new QVBoxLayout;

  addLayout->addLayout(add1Layout);
  addLayout->addLayout(add2Layout);

  add1Layout->addWidget(m_pcAddSet);
  add1Layout->addWidget(new QLabel("Qualified Sets"));
  add2Layout->addWidget(addButton);
  add2Layout->addStretch();

  //ListLayout aufbauen
  QVBoxLayout* buttonLayout = new QVBoxLayout;
  listLayout->addWidget(m_pcQualSets);
  listLayout->addLayout(buttonLayout);
  buttonLayout->addWidget(delButton);
  buttonLayout->addStretch();
  buttonLayout->addWidget(shareButton);

  //squareLayout aufbauen
  squareLayout->addWidget(m_pcForceSquare);
  squareLayout->addWidget(new QLabel("Force Square"));
  squareLayout->addStretch();
  squareLayout->addWidget(quitButton);

  //connect Signals and Slots
  //connect(exitButton, SIGNAL(clicked()), this, SLOT(quit()));
  connect(quitButton, SIGNAL(clicked()), this, SLOT(accept()));
  connect(addButton,  SIGNAL(clicked()), this, SLOT(add()));
  connect(openButton, SIGNAL(clicked()), this, SLOT(open()));
  connect(delButton,  SIGNAL(clicked()), this, SLOT(remove()));
  connect(shareButton,SIGNAL(clicked()), this, SLOT(share()));
}

GasGUI::~GasGUI()
{
}

void GasGUI::add()
{
  vector<unsigned> vTemp;
  unsigned uiTemp;
  QString sEintrag = m_pcAddSet->text();
  QStringList cNumberList = sEintrag.split(QRegExp("[^0-9]+"),
                                           QString::SkipEmptyParts);

  //Convert into numbers - and discard duplicates
  foreach(QString sNumber, cNumberList)
  {
    uiTemp = sNumber.toUInt();
    if (std::find(vTemp.begin(), vTemp.end(), uiTemp)==vTemp.end())
      vTemp.push_back(uiTemp);
  }
  //sort
  sort(vTemp.begin(), vTemp.end());
  //And back into QStringList
  cNumberList.clear();
  for (vector<unsigned>::iterator i = vTemp.begin();
       i != vTemp.end(); ++i)
  {
    cNumberList << QString::number(*i);
  }
  //join the stringlist to one string and insert it
  m_pcQualSets->addItem(cNumberList.join(","));
}

void GasGUI::open()
{
  QString sFile = QFileDialog::getOpenFileName(
                       this,
                       tr("Choose File to share"),
                       QDir::homePath(),
                       tr("Pictures (*.bmp *.png *.jpg)"));

  if (sFile!="")
  {
    m_pcFileName->clear();
    m_pcFileName->insert(sFile);
  }
}

void GasGUI::remove()
{
  QList<QListWidgetItem*> selectedList = m_pcQualSets->selectedItems();
  QList<QListWidgetItem*>::iterator i;

  //loop over all selected items in list and remove them
  for (i=selectedList.begin(); i != selectedList.end(); ++i)
    delete *i;
}

void GasGUI::share()
{
  SubSet sSubSet;
  GAS cGas;
  GasVCS* pcVCS;
  QString cFile;

  /*************** Begin Test Setup ***********************/
  sSubSet.insert(1);
  sSubSet.insert(2);
  sSubSet.insert(3);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(1);
  sSubSet.insert(2);
  sSubSet.insert(4);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(1);
  sSubSet.insert(2);
  sSubSet.insert(5);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(1);
  sSubSet.insert(3);
  sSubSet.insert(4);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(1);
  sSubSet.insert(3);
  sSubSet.insert(5);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(1);
  sSubSet.insert(4);
  sSubSet.insert(5);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(2);
  sSubSet.insert(3);
  sSubSet.insert(4);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(2);
  sSubSet.insert(3);
  sSubSet.insert(5);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(2);
  sSubSet.insert(4);
  sSubSet.insert(5);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();
  sSubSet.insert(3);
  sSubSet.insert(4);
  sSubSet.insert(5);
  cGas.AddQualifiedSet(sSubSet);
  sSubSet.clear();

  cFile="/home/kermit/projects/Examples/A.bmp";
  /*************** End Test Setup ***********************/
  //Secret image given?
/*  cFile=m_pcFileName->text();
  if (cFile=="")
  {
    QMessageBox::information(this,
                             tr("Error"),
                             tr("You have to supply a secret to share!"),
                             QMessageBox::Ok);
    return;
  }

  //construct gas
  QListWidgetItem* pcItem;
  for (unsigned i=0; (pcItem = m_pcQualSets->item(i))!=NULL; ++i)
  {
    sSubSet.clear();
    QString text(pcItem->text());
    //Text splitten
    QStringList cNumberList = text.split(QRegExp("[^0-9]+"),
                                         QString::SkipEmptyParts);
    //convert to subset
    foreach(QString sNumber, cNumberList)
      sSubSet.insert(sNumber.toUInt());
    //insert into gas
    if (cGas.AddQualifiedSet(sSubSet)==-1)
    {
      QMessageBox::information(this,
                               tr("Invalid GAS"),
                               tr("There are isolated participants (participants that "
                                  "allowed to reconstruct the secret without the help "
                                  "of any other person). This is not allowed!"));
      return;
    }
  }*/

  //Gas gueltig?
  if (cGas.Check() != 1)
  {
    QMessageBox::information(this,
                             tr("Invalid GAS"),
                             tr("There are non essential participants (are not member "
                                "in any qualified set)! This is not allowed!"));
    return;
  }

  //GasVCS konstruieren
  pcVCS = new GasVCS(cGas);
  pcVCS->SetSecretFileName(static_cast<const char*>(cFile.toAscii()));

  //Share
  QStringList* pcRet = NULL;

  switch(m_pcForceSquare->checkState())
  {
    case Qt::Unchecked:
      pcRet=pcVCS->Share(false);
      break;
    case Qt::PartiallyChecked:
      //not used here
      break;
    case Qt::Checked:
      pcRet=pcVCS->Share(true);
      break;
  }

  //Print Message
  if (pcRet != NULL)
  {
    QMessageBox::information(this, tr("Success"),
                             "The Shares\n\n"+(pcRet->join("\n"))+
                             "\n\nhave been written");
  }
  else
    QMessageBox::information(this, tr("Error"), tr("Unspecified error!"));

  //cleanup
  delete pcVCS;
}
