/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gasvcs.h"
#include "nausn.h"

#include <iostream>

using namespace std;

GasVCS::GasVCS(GAS& g) : VCryptScheme(g.GetUser())
{
  pcGAS = new GAS(g);
}

GasVCS::~GasVCS()
{
  delete pcGAS;
}

void GasVCS::CreateBaseMatrices()
{
  DMatrix* CA;
  //Die Matrizen des verwendeten k aus k VCS
  NausN* pcTempVCS;
  DMatrix* S0;
  DMatrix* S1;
  int iEl, iError;

  //CA Matrix holen
  CA = pcGAS->GetCA();
  //NausN VCS konstruieren
  pcTempVCS = new NausN(CA->GetXDim());

  //Basismatrizen fuer k aus k VCS holen
  pcTempVCS->GetBaseMatrices(&S0, &S1);

  Row cRowS0(S0->GetXDim(), 0);
  Row cRowS1(S1->GetXDim(), 0);

  for (unsigned i=m_uiUser; i > 0; i--)
  {
    //delete rows
    for (unsigned c=0;c < cRowS0.size(); c++)
    {
      cRowS0[c]=0;
      cRowS1[c]=0;
    }

    //create rows
    for (unsigned c=0;c < CA->GetXDim(); c++)
    {
      CA->GetElem(c, i-1, iEl);
      if (iEl)
      {
        cRowS0 = OrRows(cRowS0, S0->GetRow(c, iError));
        cRowS1 = OrRows(cRowS1, S1->GetRow(c, iError));
      }
    }

    //write rows to basematrices
    if (m_pcWhiteBase == NULL)
    {
      m_pcWhiteBase=new DMatrix(cRowS0.size(), 1);
      m_pcWhiteBase->SetRow(0, cRowS0);
    }
    else
    {
      m_pcWhiteBase->InsertRow(0, cRowS0);
    }

    if (m_pcBlackBase == NULL)
    {
      m_pcBlackBase = new DMatrix(cRowS1.size(), 1);
      m_pcBlackBase->SetRow(0, cRowS1);
    }
    else
    {
      m_pcBlackBase->InsertRow(0, cRowS1);
    }
  }

  //cleanup
  delete pcTempVCS;
}
