/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "imposer.h"

#include <iostream>

using namespace std;

Imposer::Imposer()
{
  m_usZoom = 1;
}

Imposer::~Imposer()
{
}

bool Imposer::SaveImposedImage(std::string sImageName)
{
  QImage* pcImage = GetImposedImage();

  //Write files (write bmps - because they are pixel oriented - we can't afford
  //any approximations)
  if (sImageName.substr(sImageName.size()-4, 4)!=".bmp")
    sImageName+=".bmp";

  //todo: Zoomen

  if (!pcImage->save(sImageName.c_str(), "BMP", 100))
  {
    delete pcImage;
    return false;
  }

  delete pcImage;
  return true;
}

QImage* Imposer::GetImposedImage()
{
  QSize cSize;
  QSize cSize2;
  QImage* pcImage;
  QImage* pcTempImage;
  unsigned farbe1, farbe2, farbe;
  list<string>::iterator i = slImageList.begin();

  if (i == slImageList.end())
    return NULL;

  //Erstes Bild oeffnen
  pcImage = new QImage(i->c_str());

  //Groesse holen
  cSize = pcImage->size();

  while((++i) != slImageList.end())
  {
    pcTempImage = new QImage(i->c_str());
    cSize2 = pcTempImage->size();
    if ((cSize.width()  != cSize2.width()) ||
        (cSize.height() != cSize2.height()))
    {
      continue;
    }
    //Ueberlagern
    for (int x = 0; x < cSize.width(); x++)
    {
      for (int y = 0; y < cSize.height(); y++)
      {
        //Farben des 1. Bildes holen
        farbe1   = pcImage->pixel(x, y);
        //Farben des 2. Bildes holen
        farbe2   = pcTempImage->pixel(x, y);
        //Ergebnisbild berechnen
        //(0 ist hier schwarz - 255 weiss - daher muss hier vorm verodern geflippt werden
        //und danach auch wieder zurueck geflippt werden
        farbe    = qRgb(~(~qRed(farbe1)   | ~qRed(farbe2)),
                        ~(~qGreen(farbe1) | ~qGreen(farbe2)),
                        ~(~qBlue(farbe1)  | ~qBlue(farbe2)));

        //Farbe neu setzen
        pcImage->setPixel(x, y, farbe);
      }
    }
    delete pcTempImage;
  }

  //todo: Zoom
  pcTempImage = new QImage(pcImage->scaledToHeight(cSize.height()*m_usZoom));
  delete pcImage;

  return pcTempImage;
}

bool Imposer::AddImage(std::string sImage)
{
  QImage* pcImage1;
  QImage* pcImage2;
  //make sure, the new image has the same resolution as the already inserted ones
  if (slImageList.size() != 0)
  {
    QSize cSize1, cSize2;
    //open images
    pcImage1 = new QImage(sImage.c_str());
    pcImage2 = new QImage((slImageList.begin())->c_str());
    //get size of images
    cSize1 = pcImage1->size();
    cSize2 = pcImage2->size();
    //close images
    delete pcImage1;
    delete pcImage2;
    //compare sizes
    if (!(cSize1==cSize2))
      return false;
  }
  //todo: Testen ob dieses Bild bereits enthalten ist (wobei zweimal ueberlagern
  //eh nichts aendert)
  slImageList.push_back(sImage);

  return true;
}

bool Imposer::RemImage(std::string sImage)
{
  bool bFound = false;
  list<string>::iterator i = slImageList.begin();

  while (i != slImageList.end())
  {
    if (*i == sImage)
    {
      bFound = true;
      i = slImageList.erase(i);
    }
    else
      ++i;
  }

  return bFound;
}

bool Imposer::RemImage(unsigned uiNumber)
{
  list<string>::iterator i;

  //Bis zur richtigen Stelle "durcharbeiten"
  for (i=slImageList.begin(); i != slImageList.end() && uiNumber; ++i, uiNumber--) ;

  //Stelle gefunden
  if (uiNumber == 0 && i != slImageList.end())
  {
    slImageList.erase(i);
    return true;
  }

  return false;
}

std::string Imposer::QueryImage(unsigned uiNumber)
{
  list<string>::iterator i = slImageList.begin();

  //Bis zur richtigen Stelle "durcharbeiten"
  for (i=slImageList.begin(); i != slImageList.end() && uiNumber; ++i, uiNumber--) ;

  if (uiNumber == 0 && i != slImageList.end())
    return *i;

  return "";
}

unsigned Imposer::QueryImageNumber()
{
  return slImageList.size();
}

unsigned short Imposer::GetZoom()
{
  return m_usZoom;
}

void Imposer::SetZoom(unsigned short usZoom)
{
  if (usZoom > 20)
    m_usZoom = 20;
  else if (usZoom==0)
    m_usZoom = 1;
  else
    m_usZoom = usZoom;
}

