/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "imposergui.h"

#include <QtGui>
#include <iostream>
using namespace std;

ImposerGUI::ImposerGUI()
{
  QHBoxLayout* mainLayout = new QHBoxLayout(this);
  QHBoxLayout* listLayout = new QHBoxLayout;
  QVBoxLayout* buttonLayout = new QVBoxLayout;

  //Layouts ineinander schachteln
  mainLayout->addLayout(listLayout);
  mainLayout->addLayout(buttonLayout);

  //Liste basteln
  pcListe = new QListWidget();
  pcListe->setSelectionMode(QAbstractItemView::MultiSelection);
  listLayout->addWidget(pcListe);

  //Create buttons for button layout
  QPushButton* addButton  = new QPushButton(tr("Add"));
  QPushButton* delButton  = new QPushButton(tr("Del"));
  QPushButton* saveButton = new QPushButton(tr("Save"));
  QPushButton* exitButton = new QPushButton(tr("Quit"));
  QSlider* zoomSlide      = new QSlider;
  zoomSlide->setObjectName("Zoom");
  zoomSlide->setMinimum(1);
  zoomSlide->setMaximum(20);
  zoomSlide->setTickInterval(5);
  zoomSlide->setTickPosition(QSlider::TicksRight);
  //build button layout
  buttonLayout->addWidget(addButton);
  buttonLayout->addWidget(delButton);
  buttonLayout->addWidget(zoomSlide);
  //buttonLayout->addStretch();
  buttonLayout->addWidget(saveButton);
  buttonLayout->addWidget(exitButton);

  //connect signals and slots
  connect(exitButton, SIGNAL(clicked()), this, SLOT(quit()));
  connect(saveButton, SIGNAL(clicked()), this, SLOT(save()));
  connect(addButton,  SIGNAL(clicked()), this, SLOT(add()));
  connect(delButton,  SIGNAL(clicked()), this, SLOT(del()));
  connect(zoomSlide,  SIGNAL(valueChanged(int)), this, SLOT(sliderChanged(int)));
  connect(pcListe,    SIGNAL(itemSelectionChanged()),this, SLOT(selectionChanged()));
}


ImposerGUI::~ImposerGUI()
{
}

void ImposerGUI::show_image()
{
  QImage* pcImage = cImpose.GetImposedImage();
  if (!pcImage)
  {
    cLabel.close();
    return;
  }

  //Bild anzeigen
  cLabel.setPixmap(QPixmap::fromImage(*pcImage));
  cLabel.adjustSize();
  cLabel.show();

  //Bild loeschen
  delete pcImage;
}

void ImposerGUI::save()
{
  QString file = QFileDialog::getSaveFileName(this,
                                              tr("Save file ..."),
                                              QDir::currentPath(),
                                              tr("Pictures (*.bmp *.png *.jpg);;All (*.*)"));
  if (file != "")
  {
    cImpose.SaveImposedImage(static_cast<const char*>(file.toAscii()));
  }
}

void ImposerGUI::add()
{
  bool bWrongSize = false;

  //Dateien auswählen
  QStringList lFiles = QFileDialog::getOpenFileNames(
                         this,
                         tr("Choose files to superimpose"),
                         QDir::homePath(),
                         tr("Pictures (*.bmp *.png *.jpg);;All (*.*)"));

  foreach(QString sFile, lFiles)
  {
    //QListWidgetItem* pcItem;
    QList<QListWidgetItem*> cItemList;

    //insert item
    if (!cImpose.AddImage(static_cast<const char*>(sFile.toAscii())))
    {
      cerr << "Fehlerhafter Dateiname: " << static_cast<const char*>(sFile.toAscii()) << endl;
      bWrongSize = true;
      continue;
    }
    pcListe->addItem(sFile);
    //find inserted item
    cItemList = pcListe->findItems(sFile, Qt::MatchExactly | Qt::MatchCaseSensitive | Qt::MatchFixedString);
    //mark all found items as selected
    foreach(QListWidgetItem* pcItem, cItemList)
    {
      pcItem->setSelected(true);
    }
  }

  //if any wrong sizes were encountered - warn here
  if (bWrongSize)
  {
    QMessageBox::information(this, tr("Wrong size"),
                           tr("Some images have not been added, because their size "
                              "does not match the size of the images already in the "
                              "list!"));
  }

  //update display
  show_image();
}

void ImposerGUI::del()
{
  //find marked entry in list
  QList<QListWidgetItem*> selectedList = pcListe->selectedItems();
  QList<QListWidgetItem*>::iterator i;

  //loop over all selected items in list
  for (i=selectedList.begin(); i != selectedList.end(); ++i)
  { //and delete them (this removes them from the list)
    cImpose.RemImage(static_cast<const char*>(((*i)->text()).toAscii()));
    delete *i;
  }

  //update display
  show_image();
}

void ImposerGUI::quit()
{
  cLabel.close();
  accept();
}

void ImposerGUI::sliderChanged(int value)
{
  //save new value
  cImpose.SetZoom(value);
  //redraw
  show_image();
}

void ImposerGUI::selectionChanged()
{
  QListWidgetItem* pcItem;

  pcItem = pcListe->currentItem();
  if (!pcItem)
    return;

  //Das heisst, das "currentItem" hat seinen Status geaendert
  if (pcItem->isSelected())
    cImpose.AddImage(static_cast<const char*>(((pcItem)->text()).toAscii()));
  else
    cImpose.RemImage(static_cast<const char*>(((pcItem)->text()).toAscii()));

  show_image();
}
