/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kausngui.h"
#include "kausn.h"

#include <QtGui>

using namespace std;

KausNGUI::KausNGUI()
{
  QVBoxLayout* mainLayout   = new QVBoxLayout(this);
  QHBoxLayout* fileLayout   = new QHBoxLayout;
  QHBoxLayout* settLayout   = new QHBoxLayout();

  //Layouts ineinander schachteln
  mainLayout->addLayout(fileLayout);
  mainLayout->addLayout(settLayout);

  //SettingLayout
  m_pcEdit = new QLineEdit;
  QPushButton* openButton  = new QPushButton(tr("Open"));

  fileLayout->addWidget(new QLabel("File"));
  fileLayout->addWidget(m_pcEdit);
  fileLayout->addWidget(openButton);

  //SettingsLayout fuellen
  m_pcForceSquare          = new QCheckBox;
  m_pcUser                 = new QSpinBox;
  m_pcNeeded               = new QSpinBox;
  QPushButton* quitButton  = new QPushButton(tr("Cancel"));
  QPushButton* shareButton = new QPushButton(tr("Share"));

  m_pcUser->setMinimum(2);
  m_pcUser->setMaximum(20);
  m_pcNeeded->setMinimum(2);
  m_pcNeeded->setMaximum(m_pcUser->value());

  settLayout->addWidget(m_pcNeeded);
  settLayout->addWidget(new QLabel("of"));
  settLayout->addWidget(m_pcUser);
  settLayout->addWidget(m_pcForceSquare);
  settLayout->addWidget(new QLabel("Force Square"));
  settLayout->addStretch();
  settLayout->addWidget(shareButton);
  settLayout->addWidget(quitButton);

  //connect signals and slots
  connect(shareButton, SIGNAL(clicked()), this, SLOT(shareNow()));
  connect(openButton,  SIGNAL(clicked()), this, SLOT(open()));
  connect(quitButton,  SIGNAL(clicked()), this, SLOT(accept()));

  //aenderungen am Wert von n koennen k beeinflussen
  connect(m_pcUser, SIGNAL(valueChanged(int)), this, SLOT(nchanged(int)));
}


KausNGUI::~KausNGUI()
{
}

void KausNGUI::nchanged(int iValue)
{
  m_pcNeeded->setMaximum(iValue);
}

void KausNGUI::shareNow()
{
  KausN* pcVCS;
  QString cFile;
  unsigned uiUser;
  unsigned uiNeeded;

  //Daten auslesen
  cFile=m_pcEdit->text();
  if (cFile=="")
  {
    QMessageBox::information(this,
                             tr("Error"),
                             tr("You have to supply a secret to share!"),
                             QMessageBox::Ok);
    return;
  }

  uiUser = m_pcUser->value();
  uiNeeded = m_pcNeeded->value();
  pcVCS = new KausN(uiNeeded, uiUser);
  pcVCS->SetSecretFileName(static_cast<const char*>(cFile.toAscii()));

  //Share!
  QStringList* pcRet = NULL;
  switch(m_pcForceSquare->checkState())
  {
    case Qt::Unchecked:
      pcRet = pcVCS->Share(false);
      break;
    case Qt::PartiallyChecked:
      //not used here
      break;
    case Qt::Checked:
      pcRet = pcVCS->Share(true);
      break;
  }

  //Print Message
  if (pcRet != NULL)
  {
    QMessageBox::information(this, tr("Success"),
                             "The Shares\n\n"+(pcRet->join("\n"))+
                             "\n\nhave been written");
  }
  else
    QMessageBox::information(this, tr("Error"), tr("Unspecified error!"));

  //cleanup
  delete pcVCS;

  accept();
}

void KausNGUI::open()
{
  QString sFile = QFileDialog::getOpenFileName(
                       this,
                       tr("Choose File to share"),
                       QDir::homePath(),
                       tr("Pictures (*.bmp *.png *.jpg)"));

  if (sFile!="")
  {
    m_pcEdit->clear();
    m_pcEdit->insert(sFile);
  }
}
