/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nausn.h"
#include <math.h>

#include <iostream>

using namespace std;

NausN::NausN(unsigned uiUser) : VCryptScheme(uiUser)
{
  CreateBaseMatrices();
}

NausN::~NausN()
{
  if (m_pcBlackBase)
  {
    delete m_pcBlackBase;
    m_pcBlackBase = NULL;
  }

  if (m_pcWhiteBase)
  {
    delete m_pcWhiteBase;
    m_pcWhiteBase = NULL;
  }
}

//Die Matrizen duerfen hier noch nicht angelegt sein
//m_uiUser und m_uiSubPixel haben bereits den korrekten Wert
void NausN::CreateBaseMatrices()
{
  unsigned uiAnzahl   = 0; //Speichert die Anzahl der 1en im aktuellen Vektor
  unsigned uiS0Index  = 0;
  unsigned uiS1Index  = 0;
  bool     bFound     = false;
  unsigned uiSubPixel = 1;

  //Teilmenge initialisieren
  uiSubPixel = uiSubPixel << (m_uiUser - 1);
  char* pcSubSet = new char[m_uiUser];
  memset(pcSubSet, 0, m_uiUser);

  //Matrizen anlegen
  m_pcBlackBase = new DMatrix(uiSubPixel, m_uiUser);
  m_pcWhiteBase = new DMatrix(uiSubPixel, m_uiUser);

  do
  {
    //Die aktulle Konfiguration hinzufuegen
    if (uiAnzahl % 2)
    { //Zu S1 hinzufuegen
      for (unsigned j = 0; j < m_uiUser; j++)
      {
        m_pcBlackBase->SetElem(uiS1Index, j, pcSubSet[j]);
      }
      uiS1Index++;
    }
    else
    { //Zu S0 hinzufuegen
      for (unsigned j = 0; j < m_uiUser; j++)
      {
        m_pcWhiteBase->SetElem(uiS0Index, j, pcSubSet[j]);
      }
      uiS0Index++;
    }

    //Naechste Teilmenge bzw. Spalte berechnen
    bFound = false;
    for (unsigned i=0; (i < m_uiUser) && (!bFound); i++)
    { //Im Prinzip wird das Array einfach als Binaerzahl aufgefasst und
      //wiederholt "1" addiert

      //Anzahl der 1en wird beim Rechnen aktualisiert
      if (pcSubSet[i] == 1)
      {
        pcSubSet[i] = 0;
        uiAnzahl--;
      }
      else
      {
        uiAnzahl++;
        pcSubSet[i] = 1;
        bFound = true;
      }
    }
    //Abbruchbedingung: Wird hier nur dann 0, wenn ein "Ueberlauf"
    //stattfindet, d.h. alle Teilmengen gebildet wurden
  } while (uiAnzahl != 0);

  //Aufraeumen
  delete[] pcSubSet;
}

