/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nausngui.h"
#include "nausn.h"

#include <iostream>

using namespace std;

NausNGUI::NausNGUI() : QDialog()
{
  QHBoxLayout* mainLayout   = new QHBoxLayout(this);
  QVBoxLayout* labelLayout  = new QVBoxLayout();
  QVBoxLayout* settLayout   = new QVBoxLayout();
  QHBoxLayout* subLayout    = new QHBoxLayout();
  QHBoxLayout* sett2Layout  = new QHBoxLayout();
  QVBoxLayout* buttonLayout = new QVBoxLayout();

  //Layouts ineinander schachteln
  mainLayout->addLayout(labelLayout);
  mainLayout->addLayout(settLayout);
  mainLayout->addLayout(buttonLayout);

  //SettingLayout
  m_pcEdit = new QLineEdit;
  m_pcForceSquare = new QCheckBox;
  QLabel* label3 = new QLabel("Force square");
  settLayout->addWidget(m_pcEdit);
  settLayout->addLayout(subLayout);
  settLayout->addLayout(sett2Layout);
  subLayout->addWidget(label3);
  subLayout->addWidget(m_pcForceSquare);
  subLayout->addStretch();

  //Labellayout fuellen
  QLabel* label1 = new QLabel("File");
  QLabel* label2 = new QLabel("User");
  labelLayout->addWidget(label1);
  labelLayout->addStretch();
  labelLayout->addWidget(label2);

  //SettingsLayout fuellen
  QPushButton* shareButton = new QPushButton(tr("Share"));
  m_pcUser = new QSpinBox;

  m_pcUser->setMinimum(2);
  m_pcUser->setMaximum(20);

  sett2Layout->addWidget(m_pcUser);
  sett2Layout->addWidget(shareButton);

  //Buttonlayout fuellen
  QPushButton* openButton  = new QPushButton(tr("Open"));
  QPushButton* quitButton  = new QPushButton(tr("Cancel"));
  //build button layout
  buttonLayout->addWidget(openButton);
  buttonLayout->addStretch();
  buttonLayout->addWidget(quitButton);

  //connect signals and slots
  connect(shareButton, SIGNAL(clicked()), this, SLOT(shareNow()));
  connect(openButton,  SIGNAL(clicked()), this, SLOT(open()));
  connect(quitButton,  SIGNAL(clicked()), this, SLOT(accept()));
}


NausNGUI::~NausNGUI()
{
  //m_pcEdit and m_pcUser are deconstructed automatically
}

void NausNGUI::shareNow()
{
  NausN* pcVCS;
  QString cFile;
  unsigned uiUser;

  //Daten auslesen
  cFile=m_pcEdit->text();
  if (cFile=="")
  {
    QMessageBox::information(this,
                             tr("Error"),
                             tr("You have to supply a secret to share!"),
                             QMessageBox::Ok);
    return;
  }

  uiUser = m_pcUser->value();
  pcVCS = new NausN(uiUser);
  pcVCS->SetSecretFileName(static_cast<const char*>(cFile.toAscii()));

  QStringList* pcRet = NULL;
  //Share!
  switch(m_pcForceSquare->checkState())
  {
    case Qt::Unchecked:
      pcRet = pcVCS->Share(false);
      break;
    case Qt::PartiallyChecked:
      //not used here
      break;
    case Qt::Checked:
      pcRet = pcVCS->Share(true);
      break;
  }

  //Print Message
  if (pcRet != NULL)
  {
    QMessageBox::information(this, tr("Success"),
                             "The Shares\n\n"+(pcRet->join("\n"))+
                             "\n\nhave been written");
  }
  else
    QMessageBox::information(this, tr("Error"), tr("Unspecified error!"));

  //cleanup
  delete pcVCS;

  accept();
}

void NausNGUI::open()
{
  QString sFile = QFileDialog::getOpenFileName(
                       this,
                       tr("Choose File to share"),
                       QDir::homePath(),
                       tr("Pictures (*.bmp *.png *.jpg)"));

  if (sFile!="")
  {
    m_pcEdit->clear();
    m_pcEdit->insert(sFile);
  }
}

