/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef VCRYPTSCHEME_H
#define VCRYPTSCHEME_H

/**
	@author Martin Apel,,, <kermit@frog>
*/

#include <string>
#include <set>

#include <QtGui>

#include "Matrix.hpp"

class VCryptScheme
{
protected:
  //Save images with this basename
  std::string m_sBaseName;
  //The image to share is located here (full path)
  std::string m_sSourceFile;

  //Die Basismatrizen
  DMatrix* m_pcWhiteBase; //S0
  DMatrix* m_pcBlackBase; //S1

  const unsigned m_uiUser;
  unsigned m_uiSubPixelX;
  unsigned m_uiSubPixelY;

  virtual void CreateBaseMatrices() = 0;
  void FindOptimalExpansion(unsigned m, unsigned &x, unsigned &y,
                            bool bForceQuadratic=false);
  int ModifyBaseMatrices(unsigned x, unsigned y, DMatrix* S0, DMatrix* S1);

  bool WritePixel(QImage* pcImage, unsigned x, unsigned y, Row cPixel,
                  QSize cSubPixelSize);
  //Verodern von Zeilen
  Row OrRows(const std::set<Row>& cRows);
  Row OrRows(const Row& r1, const Row& r2);

  //Optimize basematrices
  void Optimize();
  bool MatchCols(DMatrix* S0, unsigned uiCol0, DMatrix* S1, unsigned uiCol1,
                 unsigned uiMatchTo);
public:
  VCryptScheme(unsigned uiUser);
  virtual ~VCryptScheme();

  std::string GetBaseName();
  bool        SetBaseName(std::string sBaseName);

  std::string GetSecretFileName();
  bool        SetSecretFileName(std::string sSourceFile);

  //returns the basematrices of the VCS
  int GetBaseMatrices(DMatrix** pcWhiteBase, DMatrix** pcBlackBase);

  //returns subpixel for given white (bBlack==false==0)
  //or black (bBlack==true==1) pixel
  DMatrix* TranslatePixel(QRgb cColor);

  //The Dealer sits here
  QStringList* Share(bool bForceSquare = false);
};

#endif
