/***************************************************************************
 *   Copyright (C) 2007 by Martin Apel,,,                                  *
 *   kermit@frog                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <QtGui>
#include "viscrypt.h"
#include "imposer.h"
#include "imposergui.h"
#include "nausn.h"
#include "gas.h"
#include "gasvcs.h"

#include <iostream>

using namespace std;

VisCrypt::VisCrypt()
{
  pcLeinwand = new QLabel(this);
  setCentralWidget(pcLeinwand);

  CreateActions();
  CreateMenus();

  QPoint pos(200, 200);
  QSize size(150, 300);
  resize(size);
  move(pos);

  //create buttons
  QPushButton* pcKausKButton  = new QPushButton("K out of K");
  QPushButton* pcKausNButton  = new QPushButton("K out of N");
  QPushButton* pcGasButton    = new QPushButton("GAS-VCS");
  QPushButton* pcImposeButton = new QPushButton("Superimpose");

  //insert buttons
  QVBoxLayout* mainLayout = new QVBoxLayout(pcLeinwand);
  mainLayout->addWidget(pcKausKButton);
  mainLayout->addWidget(pcKausNButton);
  mainLayout->addWidget(pcGasButton);
  mainLayout->addWidget(pcImposeButton);

  //connect buttons
  connect(pcKausKButton,  SIGNAL(clicked()), this, SLOT(KausK()));
  connect(pcKausNButton,  SIGNAL(clicked()), this, SLOT(KausN()));
  connect(pcGasButton,    SIGNAL(clicked()), this, SLOT(Gas()));
  connect(pcImposeButton, SIGNAL(clicked()), this, SLOT(SuperImpose()));
}

void VisCrypt::SuperImpose()
{
  cImposeGui.show();
}

void VisCrypt::KausK()
{
  cNausNGui.show();
}

void VisCrypt::KausN()
{
  cKausNGui.show();
}

void VisCrypt::Gas()
{
  cGasGui.show();
}

void VisCrypt::About()
{
  QMessageBox::about(this, tr("About VisCrypt"),
    tr("VisCrypt helps you to share images by using techniques called "
       "<b>Visual Cryptography</b>. It has been written by Martin Apel."));
}

void VisCrypt::CreateActions()
{
  exitAct = new QAction(tr("&Exit"), this);
  exitAct->setStatusTip(tr("Exits the program"));
  connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

  aboutAct = new QAction(tr("&About"), this);
  aboutAct->setStatusTip(tr("Show the application's About box"));
  connect(aboutAct, SIGNAL(triggered()), this, SLOT(About()));

  aboutQtAct = new QAction(tr("About &Qt"), this);
  aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
  connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

  kauskAct = new QAction(tr("&k out of k"), this);
  connect(kauskAct, SIGNAL(triggered()), this, SLOT(KausK()));

  kausnAct = new QAction(tr("k out of &n"), this);
  connect(kausnAct, SIGNAL(triggered()), this, SLOT(KausN()));

  gasAct   = new QAction(tr("&GAS"),this);
  connect(gasAct, SIGNAL(triggered()), this, SLOT(Gas()));

  superImposeAct = new QAction(tr("&SuperImpose ..."), this);
  connect(superImposeAct, SIGNAL(triggered()), this, SLOT(SuperImpose()));
}

void VisCrypt::CreateMenus()
{
  fileMenu = menuBar()->addMenu(tr("&Mode"));
  fileMenu->addAction(kauskAct);
  fileMenu->addAction(kausnAct);
  fileMenu->addAction(gasAct);
  fileMenu->addAction(superImposeAct);
  fileMenu->addAction(exitAct);

  menuBar()->addSeparator();

  helpMenu = menuBar()->addMenu(tr("&Help"));
  helpMenu->addAction(aboutAct);
  helpMenu->addAction(aboutQtAct);
}

VisCrypt::~VisCrypt()
{
}
